@@ -9,11 +9,11 @@
 #include <pthread.h>
 
 #include "libbalsa.h"
+#include "libbalsa-conf.h"
 #include "imap-handle.h"
 #include "imap-server.h"
 #include "imap-commands.h"
-
-#include <libgnome/libgnome.h> 
+#include "i18n.h"
 
 #ifdef USE_TLS
 #define REQ_SSL(s) (LIBBALSA_SERVER(s)->use_ssl)
@@ -21,6 +21,9 @@
 #define REQ_SSL(s) (0)
 #endif
 
+/** wait 20 seconds for packets */
+#define IMAP_CMD_TIMEOUT (20*1000)
+
 static LibBalsaServerClass *parent_class = NULL;
 
 struct LibBalsaImapServer_ {
@@ -40,6 +43,8 @@ struct LibBalsaImapServer_ {
                                     sessions. If FALSE, messages will be
                                     kept in /tmp and cleaned on exit. */
     unsigned has_fetch_bug:1;
+    unsigned use_status:1; /**< server has fast STATUS command */
+    unsigned use_idle:1;  /**< IDLE will work: no dummy firewall on the way */
 };
 
 typedef struct LibBalsaImapServerClass_ {
@@ -52,8 +57,8 @@ static void libbalsa_imap_server_init(LibBalsaImapServer * server);
 static void libbalsa_imap_server_finalize(GObject * object);
 static gboolean connection_cleanup(gpointer ptr);
 
-/* Poll every 5 minutes */
-#define CONNECTION_CLEANUP_POLL_PERIOD  (5*60)
+/* Poll every 5 minutes - must be shortest ouf of the times here. */
+#define CONNECTION_CLEANUP_POLL_PERIOD  (2*60)
 /* Cleanup connections more then 10 minutes idle */
 #define CONNECTION_CLEANUP_IDLE_TIME    (10*60)
 /* Send NOOP after 20 minutes to keep a connection alive */
@@ -61,7 +66,7 @@ static gboolean connection_cleanup(gpointer ptr);
 /* We try to avoid too many connections per server */
 #define MAX_CONNECTIONS_PER_SERVER 20
 
-G_LOCK_DEFINE_STATIC(imap_servers);
+static pthread_mutex_t imap_servers_lock = PTHREAD_MUTEX_INITIALIZER;
 static GHashTable *imap_servers = NULL;
 
 struct handle_info {
@@ -112,26 +117,27 @@ static void libbalsa_imap_server_set_username(LibBalsaServer * server,
     if(server->host && name) { /* we have been initialized... */
         LibBalsaImapServer *imap_server = LIBBALSA_IMAP_SERVER(server);
         
-        G_LOCK(imap_servers);
+        pthread_mutex_lock(&imap_servers_lock);
         g_hash_table_steal(imap_servers, imap_server->key);
         g_free(imap_server->key);
         imap_server->key = g_strdup_printf("%s@%s", name, server->host);
         g_hash_table_insert(imap_servers, imap_server->key, imap_server);
-        G_UNLOCK(imap_servers);
+        pthread_mutex_unlock(&imap_servers_lock);
     }
     (parent_class)->set_username(server, name);
 }
-static void libbalsa_imap_server_set_host(LibBalsaServer * server,
-                      const gchar * host, gboolean use_ssl)
+static void
+libbalsa_imap_server_set_host(LibBalsaServer * server,
+                              const gchar * host, gboolean use_ssl)
 {
     if(server->user && host) { /* we have been initialized... */
         LibBalsaImapServer *imap_server = LIBBALSA_IMAP_SERVER(server);
-        G_LOCK(imap_servers);
+        pthread_mutex_lock(&imap_servers_lock);
         g_hash_table_steal(imap_servers, imap_server->key);
         g_free(imap_server->key);
         imap_server->key = g_strdup_printf("%s@%s", server->user, host);
         g_hash_table_insert(imap_servers, imap_server->key, imap_server);
-        G_UNLOCK(imap_servers);
+        pthread_mutex_unlock(&imap_servers_lock);
     }
     (parent_class)->set_host(server, host, use_ssl);
 }
@@ -151,7 +157,6 @@ libbalsa_imap_server_class_init(LibBalsaImapServerClass * klass)
     server_class->set_username = libbalsa_imap_server_set_username;
     server_class->set_host = libbalsa_imap_server_set_host;
 #if 0
-    klass->set_password = libbalsa_imap_server_real_set_password;
     klass->get_password = NULL; /* libbalsa_imap_server_real_get_password; */
 #endif
 }
@@ -159,8 +164,6 @@ libbalsa_imap_server_class_init(LibBalsaImapServerClass * klass)
 static void
 libbalsa_imap_server_init(LibBalsaImapServer * imap_server)
 {
-    LibBalsaServer *server = LIBBALSA_SERVER(imap_server);
-    server->type = LIBBALSA_SERVER_IMAP;
     imap_server->key = NULL;
     imap_server->lock = g_mutex_new();
     imap_server->max_connections = MAX_CONNECTIONS_PER_SERVER;
@@ -172,6 +175,7 @@ libbalsa_imap_server_init(LibBalsaImapServer * imap_server)
 #else
     imap_server->persistent_cache = TRUE;
 #endif /* ENABLE_TOUCH_UI */
+    imap_server->use_idle = TRUE;
     imap_server->connection_cleanup_id = 
         g_timeout_add(CONNECTION_CLEANUP_POLL_PERIOD*1000,
                       connection_cleanup, imap_server);
@@ -189,9 +193,9 @@ libbalsa_imap_server_finalize(GObject * object)
     server = LIBBALSA_SERVER(object);
     imap_server = LIBBALSA_IMAP_SERVER(object);
 
-    G_LOCK(imap_servers);
+    pthread_mutex_lock(&imap_servers_lock);
     g_hash_table_remove(imap_servers, imap_server->key);
-    G_UNLOCK(imap_servers);
+    pthread_mutex_unlock(&imap_servers_lock);
     
     g_source_remove(imap_server->connection_cleanup_id);
 
@@ -285,7 +289,7 @@ lb_imap_server_info_new(LibBalsaServer *server)
      * libbalsa_server_user_cb(). */
 
     handle = imap_mbox_handle_new();
-    imap_handle_set_timeout(handle, 30000); /* wait 30 seconds for packets */
+    imap_handle_set_timeout(handle, IMAP_CMD_TIMEOUT);
     info = g_new0(struct handle_info, 1);
     info->handle = handle;
     imap_handle_set_monitorcb(handle, monitor_cb, info);
@@ -298,6 +302,11 @@ lb_imap_server_info_new(LibBalsaServer *server)
     case LIBBALSA_TLS_REQUIRED: mode = IMAP_TLS_REQUIRED; break;
     }
     imap_handle_set_tls_mode(handle, mode);
+    imap_handle_set_option(handle, IMAP_OPT_ANONYMOUS, server->try_anonymous);
+    imap_handle_set_option(handle, IMAP_OPT_CLIENT_SORT, TRUE);
+    imap_handle_set_option(handle, IMAP_OPT_BINARY, TRUE);
+    imap_handle_set_option(handle, IMAP_OPT_IDLE,
+                           LIBBALSA_IMAP_SERVER(server)->use_idle);
     return info;
 }
 
@@ -344,8 +353,11 @@ lb_imap_server_cleanup(LibBalsaImapServer * imap_server)
 
     for (list = imap_server->used_handles; list; list = list->next) {
         struct handle_info *info = list->data;
-
-        if (info->last_used < idle_marker) {
+        /* We poll selected handles each time (unless IDLE is on
+           already).  Remaining handles are just kept alive. */ 
+        if ( (imap_mbox_is_selected(info->handle) &&
+              !imap_server->use_idle) || 
+            info->last_used < idle_marker) {
             /* ignore errors here - the point is to keep the
                connection alive and if there is no connection, noop
                will be, well, no-op. Other operations may possibly
@@ -385,15 +397,15 @@ static LibBalsaImapServer* get_or_create(const gchar *username,
     gchar *key;
 
     if (!imap_servers) {
-        G_LOCK(imap_servers);
+        pthread_mutex_lock(&imap_servers_lock);
         if (!imap_servers)
             imap_servers = g_hash_table_new(g_str_hash, g_str_equal);
-        G_UNLOCK(imap_servers);
+        pthread_mutex_unlock(&imap_servers_lock);
     }
 
     /* lookup username@host */
     key = g_strdup_printf("%s@%s", username, host);
-    G_LOCK(imap_servers);
+    pthread_mutex_lock(&imap_servers_lock);
     imap_server = g_hash_table_lookup(imap_servers, key);
     if (!imap_server) {
         imap_server = g_object_new(LIBBALSA_TYPE_IMAP_SERVER, NULL);
@@ -403,7 +415,7 @@ static LibBalsaImapServer* get_or_create(const gchar *username,
         g_free(key);
         g_object_ref(imap_server);
     }
-    G_UNLOCK(imap_servers);
+    pthread_mutex_unlock(&imap_servers_lock);
     return imap_server;
 }
 
@@ -431,17 +443,17 @@ libbalsa_imap_server_new_from_config(void)
     gboolean d, d1;
     gint tls_mode, conn_limit;
 
-    tmp_server.host = gnome_config_get_string("Server");
+    tmp_server.host = libbalsa_conf_get_string("Server");
     if(strrchr(tmp_server.host, ':') == NULL) {
         gint port;
-        port = gnome_config_get_int_with_default("Port", &d);
+        port = libbalsa_conf_get_int_with_default("Port", &d);
         if (!d) {
             gchar *newhost = g_strdup_printf("%s:%d", tmp_server.host, port);
             g_free(tmp_server.host);
             tmp_server.host = newhost;
         }
     }       
-    tmp_server.user = gnome_config_private_get_string("Username");
+    tmp_server.user = libbalsa_conf_private_get_string("Username");
     if (!tmp_server.user)
         tmp_server.user = g_strdup(getenv("USER"));
 
@@ -454,19 +466,25 @@ libbalsa_imap_server_new_from_config(void)
         server->user = tmp_server.user;
         server->host = tmp_server.host;
     }
-    server->use_ssl |= gnome_config_get_bool("SSL=false");
-    tls_mode = gnome_config_get_int_with_default("TLSMode", &d);
+    d1 = libbalsa_conf_get_bool_with_default("Anonymous", &d);
+    if(!d) server->try_anonymous = !!d1;
+    server->use_ssl |= libbalsa_conf_get_bool("SSL=false");
+    tls_mode = libbalsa_conf_get_int_with_default("TLSMode", &d);
     if(!d) server->tls_mode = tls_mode;
-    conn_limit = gnome_config_get_int_with_default("ConnectionLimit", &d);
+    conn_limit = libbalsa_conf_get_int_with_default("ConnectionLimit", &d);
     if(!d) imap_server->max_connections = conn_limit;
-    d1 = gnome_config_get_bool_with_default("PersistentCache", &d);
+    d1 = libbalsa_conf_get_bool_with_default("PersistentCache", &d);
     if(!d) imap_server->persistent_cache = !!d1;
-    d1 = gnome_config_get_bool_with_default("HasFetchBug", &d);
+    d1 = libbalsa_conf_get_bool_with_default("HasFetchBug", &d);
     if(!d) imap_server->has_fetch_bug = !!d1;
+    d1 = libbalsa_conf_get_bool_with_default("UseStatus", &d);
+    if(!d) imap_server->use_status = !!d1;
+    d1 = libbalsa_conf_get_bool_with_default("UseIdle", &d);
+    if(!d) imap_server->use_idle = !!d1;
     if (!server->passwd) {
-        server->remember_passwd = gnome_config_get_bool("RememberPasswd=false");
+        server->remember_passwd = libbalsa_conf_get_bool("RememberPasswd=false");
         if(server->remember_passwd)
-            server->passwd = gnome_config_private_get_string("Password");
+            server->passwd = libbalsa_conf_private_get_string("Password");
         if(server->passwd && server->passwd[0] == '\0') {
             g_free(server->passwd);
             server->passwd = NULL;
@@ -485,9 +503,11 @@ void
 libbalsa_imap_server_save_config(LibBalsaImapServer *server)
 {
     libbalsa_server_save_config(LIBBALSA_SERVER(server));
-    gnome_config_set_int("ConnectionLimit", server->max_connections);
-    gnome_config_set_bool("PersistentCache", server->persistent_cache);
-    gnome_config_set_bool("HasFetchBug", server->has_fetch_bug);
+    libbalsa_conf_set_int("ConnectionLimit", server->max_connections);
+    libbalsa_conf_set_bool("PersistentCache", server->persistent_cache);
+    libbalsa_conf_set_bool("HasFetchBug", server->has_fetch_bug);
+    libbalsa_conf_set_bool("UseStatus",   server->use_status);
+    libbalsa_conf_set_bool("UseIdle",     server->use_idle);
 }
 
 /* handle_connection_error() releases handle_info data, clears password
@@ -497,17 +517,24 @@ handle_connection_error(int rc, struct handle_info *info,
                         LibBalsaServer *server, GError **err)
 {
     gchar *msg = imap_mbox_handle_get_last_msg(info->handle);
-    if(rc == IMAP_AUTH_FAILURE)
-        libbalsa_server_set_password(server, NULL);
-    if(rc != IMAP_CONNECT_FAILED) {
-        g_set_error(err, LIBBALSA_MAILBOX_ERROR,
-                    LIBBALSA_MAILBOX_AUTH_ERROR,
-                    _("Cannot connect to the server: %s"), msg);
-    } else
+    switch(rc) {
+    case IMAP_AUTH_FAILURE:
+        libbalsa_server_set_password(server, NULL); break;
+    case IMAP_CONNECT_FAILED:
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_NETWORK_ERROR,
                     _("Cannot connect to the server"));
-    
+        break;
+    case IMAP_AUTH_CANCELLED:
+        g_set_error(err, LIBBALSA_MAILBOX_ERROR,
+                    LIBBALSA_MAILBOX_AUTH_CANCELLED,
+                    _("Cannot connect to the server: %s"), msg);
+        break;
+    default:
+        g_set_error(err, LIBBALSA_MAILBOX_ERROR,
+                    LIBBALSA_MAILBOX_AUTH_ERROR,
+                    _("Cannot connect to the server: %s"), msg);
+    }    
     g_free(msg);
     lb_imap_server_info_free(info);
 }
@@ -785,10 +812,10 @@ static void close_all_connections_cb(gpointer key, gpointer value,
 void
 libbalsa_imap_server_close_all_connections(void)
 {
-    G_LOCK(imap_servers);
-    if(imap_servers) 
+    pthread_mutex_lock(&imap_servers_lock);
+    if (imap_servers)
         g_hash_table_foreach(imap_servers, close_all_connections_cb, NULL);
-    G_UNLOCK(imap_servers);
+    pthread_mutex_unlock(&imap_servers_lock);
     libbalsa_imap_purge_temp_dir(0);
 }
 
@@ -853,3 +880,37 @@ libbalsa_imap_server_has_bug(LibBalsaImapServer *server,
 {
     return server->has_fetch_bug;
 }
+
+void
+libbalsa_imap_server_set_use_status(LibBalsaImapServer *server, 
+                                    gboolean use_status)
+{
+    server->use_status = !!use_status;
+}
+gboolean
+libbalsa_imap_server_get_use_status(LibBalsaImapServer *server)
+{
+    return server->use_status;
+}
+
+void
+libbalsa_imap_server_set_use_idle(LibBalsaImapServer *server, 
+                                  gboolean use_idle)
+{
+    GList *list;
+
+    server->use_idle = !!use_idle;
+    printf("Server will%s use IDLE\n",
+           server->use_idle ? "" : " NOT");
+    for (list = server->used_handles; list; list = list->next) {
+        struct handle_info *info = list->data;
+        imap_handle_set_option(info->handle, IMAP_OPT_IDLE,
+                               server->use_idle);
+    }
+}
+
+gboolean
+libbalsa_imap_server_get_use_idle(LibBalsaImapServer *server)
+{
+    return server->use_idle;
+}
