@@ -803,7 +803,12 @@ bmbl_do_popup(GtkTreeView * tree_view, GtkTreePath * path,
 
     menu = balsa_mailbox_node_get_context_menu(mbnode);
     g_object_ref(menu);
+#if GLIB_CHECK_VERSION(2, 10, 0)
+    g_object_ref_sink(menu);
+#else                           /* GLIB_CHECK_VERSION(2, 10, 0) */
+    g_object_ref(menu);
     gtk_object_sink(GTK_OBJECT(menu));
+#endif                          /* GLIB_CHECK_VERSION(2, 10, 0) */
     gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
 		   event_button, event_time);
     g_object_unref(menu);
@@ -866,16 +871,20 @@ bmbl_drag_cb(GtkWidget * widget, GdkDragContext * context,
     LibBalsaMailbox *mailbox;
     LibBalsaMailbox *orig_mailbox;
     BalsaIndex *orig_index;
+    GArray *selected;
 
     if (!selection_data || !selection_data->data)
 	/* Drag'n'drop is weird... */
 	return;
 
     orig_index = *(BalsaIndex **) selection_data->data;
-    if (orig_index->selected->len == 0)
+    selected = balsa_index_selected_msgnos_new(orig_index);
+    if (selected->len == 0) {
 	/* it is actually possible to drag from GtkTreeView when no rows
 	 * are selected: Disable preview for that. */
-       return; 
+        balsa_index_selected_msgnos_free(orig_index, selected);
+        return; 
+    }
 
     orig_mailbox = orig_index->mailbox_node->mailbox;
 
@@ -896,10 +905,11 @@ bmbl_drag_cb(GtkWidget * widget, GdkDragContext * context,
 
         /* cannot transfer to the originating mailbox */
         if (mailbox != NULL && mailbox != orig_mailbox)
-            balsa_index_transfer(orig_index, orig_index->selected, mailbox,
+            balsa_index_transfer(orig_index, selected, mailbox,
                                  context->action != GDK_ACTION_MOVE);
         gtk_tree_path_free(path);
     }
+    balsa_index_selected_msgnos_free(orig_index, selected);
 
     if (balsa_find_iter_by_data(&iter, orig_mailbox))
         gtk_tree_selection_select_iter(selection, &iter);
@@ -2291,13 +2301,13 @@ balsa_mblist_set_status_bar(LibBalsaMailbox * mailbox)
 
     /* xgettext: this is the first part of the message
      * "Shown mailbox: %s with %d messages, %d new, %d hidden". */
-    g_string_append_printf(desc, _("Shown mailbox: %s"), mailbox->name);
+    g_string_append_printf(desc, _("Shown mailbox: %s "), mailbox->name);
     if (total_messages > 0) {
         /* xgettext: this is the second part of the message
          * "Shown mailbox: %s with %d messages, %d new, %d hidden". */
         g_string_append_printf(desc,
-                               ngettext(" with %d message",
-                                        " with %d messages",
+                               ngettext("with %d message",
+                                        "with %d messages",
                                         total_messages), total_messages);
         if (unread_messages > 0)
             /* xgettext: this is the third part of the message
